package com.bonus.material.basic.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import cn.hutool.http.HttpException;
import com.alibaba.fastjson.JSONObject;
import com.bonus.common.log.enums.OperaType;
import com.bonus.material.common.annotation.PreventRepeatSubmit;
import com.bonus.material.common.utils.HttpClient;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.bonus.common.log.annotation.SysLog;
import com.bonus.common.security.annotation.RequiresPermissions;
import com.bonus.material.basic.domain.BmProject;
import com.bonus.material.basic.service.IBmProjectService;
import com.bonus.common.core.web.controller.BaseController;
import com.bonus.common.core.web.domain.AjaxResult;
import com.bonus.common.core.utils.poi.ExcelUtil;
import com.bonus.common.core.web.page.TableDataInfo;

/**
 * 标段工程管理Controller
 * 
 * @author xsheng
 * @date 2024-09-27
 */
@Api(tags = "标段工程管理接口")
@RestController
@RequestMapping("/bm_project")
public class BmProjectController extends BaseController
{

    @Autowired
    private IBmProjectService bmProjectService;

    /**
     * 查询标段工程管理列表
     */
    @ApiOperation(value = "查询标段工程管理列表")
    @RequiresPermissions("basic:project:list")
    @GetMapping("/list")
    public TableDataInfo list(BmProject bmProject)
    {
        startPage();
        List<BmProject> list = bmProjectService.selectBmProjectList(bmProject);
        return getDataTable(list);
    }

    /**
     * 导出标段工程管理列表
     */
    @ApiOperation(value = "导出标段工程管理列表")
    @PreventRepeatSubmit
    @RequiresPermissions("basic:project:export")
    @SysLog(title = "标段工程管理", businessType = OperaType.EXPORT, logType = 1,module = "仓储管理->导出标段工程管理")
    @PostMapping("/export")
    public void export(HttpServletResponse response, BmProject bmProject)
    {
        List<BmProject> list = bmProjectService.selectBmProjectList(bmProject);
        ExcelUtil<BmProject> util = new ExcelUtil<BmProject>(BmProject.class);
        util.exportExcel(response, list, "标段工程管理数据");
    }

    /**
     * 获取标段工程管理详细信息
     */
    @ApiOperation(value = "获取标段工程管理详细信息")
    @RequiresPermissions("basic:project:query")
    @GetMapping(value = "/{proId}")
    public AjaxResult getInfo(@PathVariable("proId") Long proId)
    {
        return success(bmProjectService.selectBmProjectByProId(proId));
    }

    /**
     * 新增标段工程管理
     */
    @ApiOperation(value = "新增标段工程管理")
    @PreventRepeatSubmit
    @RequiresPermissions("basic:project:add")
    @SysLog(title = "标段工程管理", businessType = OperaType.INSERT, logType = 1,module = "仓储管理->新增标段工程管理")
    @PostMapping
    public AjaxResult add(@Validated @RequestBody BmProject bmProject)
    {
        return bmProjectService.insertBmProject(bmProject);
    }

    /**
     * 修改标段工程管理
     */
    @ApiOperation(value = "修改标段工程管理")
    @PreventRepeatSubmit
    @RequiresPermissions("basic:project:edit")
    @SysLog(title = "标段工程管理", businessType = OperaType.UPDATE, logType = 1,module = "仓储管理->修改标段工程管理")
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody BmProject bmProject)
    {
        return bmProjectService.updateBmProject(bmProject);
    }

    /**
     * 删除标段工程管理
     */
    @ApiOperation(value = "删除标段工程管理")
    @PreventRepeatSubmit
    @RequiresPermissions("basic:project:remove")
    @SysLog(title = "标段工程管理", businessType = OperaType.DELETE, logType = 1,module = "仓储管理->删除标段工程管理")
	@DeleteMapping("/{proId}")
    public AjaxResult remove(@PathVariable Long proId)
    {
        return bmProjectService.deleteBmProjectByProId(proId);
    }


    @ApiOperation(value = "从第三方获取i8工程信息")
    @GetMapping("/getProjectList")
    public AjaxResult getProjectList(String token)
    {
        try {
            String url =  "http://10.138.55.105:8097/data-center/dc-base/api/project/list";
            //String url =  "http://192.168.1.167:8097/data-center/dc-base/api/project/list";
            String res = HttpClient.sendGet(url,token);
            JSONObject jsonObject = JSONObject.parseObject(res);
            return AjaxResult.success(jsonObject);
        } catch (HttpException e) {
            return AjaxResult.error(e.getMessage());
        }
    }
}
